/*
Copyright (C) Cay S. Horstmann 2017

This file is part of the Big C++ media utilities.

The Big C++ media utilities are free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

The Big C++ media utilities are distributed in the hope that they will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with the Big C++ media utilities. If not, see <https://www.gnu.org/licenses/>. 
*/

#include "animation.h"
#include <algorithm>

Animation::Animation(string filename, int width, int height) : _frame(width * height * 4)
{
   _width = width;
   _height = height;
   fill(_frame.begin(), _frame.end(), 255); // fill with white
   if (filename.substr(filename.length() - 4) == ".gif")
   {
      _gif = jo_gif_start(filename.c_str(), width, height, 0, 32);
   }
}

void Animation::add(const Picture& pic, int x, int y)
{
   for (int py = 0; py < pic._height; py++)
      for (int px = 0; px < pic._width; px++)
      {
         int fx = x + px;
         int fy = y + py;
         if (0 <= fx && fx < _width && 0 <= fy && fy < _height)
         {
            int foff = (fy * _width + fx) * 4;
            int poff = (py * pic._width + px) * 4;
            _frame[foff] = pic._values[poff];
            _frame[foff + 1] = pic._values[poff + 1];
            _frame[foff + 2] = pic._values[poff + 2];            
         }
      }
}

void Animation::frame()
{
   jo_gif_frame(&_gif, _frame.data(), 4, true);
   fill(_frame.begin(), _frame.end(), 255); // fill with white
}

Animation::~Animation()
{
   jo_gif_end(&_gif);
}

